@@ -19,6 +19,9 @@
 #include "DiscoverVersion.h"
 #include <QTextStream>
 #include <QStandardPaths>
+#include <kstartupinfo.h>
+
+#include <QX11Info>
 
 typedef QHash<QString, DiscoverObject::CompactMode> StringCompactMode;
 Q_GLOBAL_STATIC_WITH_ARGS(StringCompactMode, s_decodeCompactMode, (StringCompactMode {
@@ -79,11 +82,13 @@ void processArgs(QCommandLineParser* parser, DiscoverObject* mainWindow)
 
 int main(int argc, char** argv)
 {
+    // needs to be set before we create the QGuiApplication
+    QCoreApplication::setAttribute(Qt::AA_DisableSessionManager, true);
+
     QApplication app(argc, argv);
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("plasmadiscover")));
     app.setAttribute(Qt::AA_DontCreateNativeWidgetSiblings);
     app.setAttribute(Qt::AA_UseHighDpiPixmaps, true);
-    app.setAttribute(Qt::AA_DisableSessionManager, true);
     KCrash::initialize();
     KQuickAddons::QtQuickSettings::init();
     KLocalizedString::setApplicationDomain("plasma-discover");
@@ -133,10 +138,18 @@ int main(int argc, char** argv)
         }
         QObject::connect(&app, &QCoreApplication::aboutToQuit, mainWindow, &DiscoverObject::deleteLater);
         QObject::connect(service, &KDBusService::activateRequested, mainWindow, [mainWindow](const QStringList &arguments, const QString &/*workingDirectory*/){
+
+
             if (!mainWindow->rootObject())
                 QCoreApplication::instance()->quit();
 
-            mainWindow->rootObject()->raise();
+
+            auto window = qobject_cast<QWindow*>(mainWindow->rootObject());
+            if (window && QX11Info::isPlatformX11()) {
+                KStartupInfo::setNewStartupId(window, QX11Info::nextStartupId());
+            }
+            window->raise();
+
             if (arguments.isEmpty())
                 return;
             QScopedPointer<QCommandLineParser> parser(createParser());
