@@ -1,23 +1,10 @@
-/***************************************************************************
- *   Copyright © 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2017 Jan Grulich <jgrulich@redhat.com>                    *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2017 Jan Grulich <jgrulich@redhat.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
+
 
 #include "FlatpakResource.h"
 #include "FlatpakBackend.h"
@@ -362,7 +349,8 @@ QString FlatpakResource::sizeDescription()
 {
     KFormat f;
     if (!isInstalled() || canUpgrade()) {
-        if (propertyState(DownloadSize) == NotKnownYet || propertyState(InstalledSize) == NotKnownYet) {
+        if (propertyState(DownloadSize) == NotKnownYet || propertyState(InstalledSize) == NotKnownYet || propertyState(DownloadSize) == Fetching || propertyState(InstalledSize) == Fetching) {
+            qobject_cast<FlatpakBackend*>(backend())->updateAppSize(this);
             return i18n("Retrieving size information");
         } else if (propertyState(DownloadSize) == UnknownOrFailed || propertyState(InstalledSize) == UnknownOrFailed) {
             return i18n("Unknown size");
@@ -370,7 +358,7 @@ QString FlatpakResource::sizeDescription()
             return i18nc("@info app size", "%1 to download, %2 on disk", f.formatByteSize(downloadSize()), f.formatByteSize(installedSize()));
         }
     } else {
-        if (propertyState(InstalledSize) == NotKnownYet) {
+        if (propertyState(InstalledSize) == NotKnownYet || propertyState(InstalledSize) == Fetching) {
             return i18n("Retrieving size information");
         } else if (propertyState(InstalledSize) == UnknownOrFailed) {
             return i18n("Unknown size");
@@ -545,7 +533,8 @@ QString FlatpakResource::installationPath() const
 QString FlatpakResource::installationPath(FlatpakInstallation* flatpakInstallation)
 {
     g_autoptr(GFile) path = flatpak_installation_get_path(flatpakInstallation);
-    return QString::fromUtf8(g_file_get_path(path));
+    g_autofree char *path_str = g_file_get_path(path);
+    return QString::fromUtf8(path_str);
 }
 
 QString FlatpakResource::installPath() const
