@@ -1,23 +1,9 @@
-/***************************************************************************
- *   Copyright © 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>       *
- *   Copyright © 2018 Abhijeet Sharma <sharma.abhijeet2096@gmail.com>      *
- *                                                                         *
- *   This program is free software; you can redistribute it and/or         *
- *   modify it under the terms of the GNU General Public License as        *
- *   published by the Free Software Foundation; either version 2 of        *
- *   the License or (at your option) version 3 or any later version        *
- *   accepted by the membership of KDE e.V. (or its successor approved     *
- *   by the membership of KDE e.V.), which shall act as a proxy            *
- *   defined in Section 14 of version 3 of the license.                    *
- *                                                                         *
- *   This program is distributed in the hope that it will be useful,       *
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *   GNU General Public License for more details.                          *
- *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- ***************************************************************************/
+/*
+ *   SPDX-FileCopyrightText: 2013 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+ *   SPDX-FileCopyrightText: 2018 Abhijeet Sharma <sharma.abhijeet2096@gmail.com>
+ *
+ *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
+ */
 
 #include "FwupdBackend.h"
 #include "FwupdResource.h"
@@ -27,7 +13,7 @@
 #include <resources/SourcesModel.h>
 #include <Transaction/Transaction.h>
 
-#include <QtConcurrent>
+#include <QCoreApplication>
 #include <KAboutData>
 #include <KLocalizedString>
 #include <KPluginFactory>
@@ -42,6 +28,7 @@ FwupdBackend::FwupdBackend(QObject* parent)
     , m_updater(new StandardBackendUpdater(this))
     , m_cancellable(g_cancellable_new())
 {
+    fwupd_client_set_user_agent_for_package(client, "plasma-discover", "1.0");
     connect(m_updater, &StandardBackendUpdater::updatesCountChanged, this, &FwupdBackend::updatesCountChanged);
 
     SourcesModel::global()->addSourcesBackend(new FwupdSourcesBackend(this));
@@ -63,15 +50,12 @@ QMap<GChecksumType, QCryptographicHash::Algorithm> FwupdBackend::gchecksumToQChr
 FwupdBackend::~FwupdBackend()
 {
     g_cancellable_cancel(m_cancellable);
-    if (!m_threadPool.waitForDone(200))
-        qWarning("Could not stop all fwupd threads");
-    m_threadPool.clear();
     g_object_unref(m_cancellable);
 
     g_object_unref(client);
 }
 
-void FwupdBackend::addResourceToList(FwupdResource* res)
+void FwupdBackend::addResource(FwupdResource* res)
 {
     res->setParent(this);
     auto &r = m_resources[res->packageName()];
@@ -83,24 +67,10 @@ void FwupdBackend::addResourceToList(FwupdResource* res)
     Q_ASSERT(m_resources.value(res->packageName()) == res);
 }
 
-FwupdResource * FwupdBackend::createDevice(FwupdDevice *device)
-{
-    const QString name = QString::fromUtf8(fwupd_device_get_name(device));
-    FwupdResource* res = new FwupdResource(name, this);
-
-    const QString deviceID = QString::fromUtf8(fwupd_device_get_id(device));
-    res->setId(QStringLiteral("org.fwupd.%1.device").arg(QString(deviceID).replace(QLatin1Char('/'),QLatin1Char('_'))));
-    res->setDeviceId(deviceID);
-    res->setDeviceDetails(device);
-    return res;
-}
-
 FwupdResource * FwupdBackend::createRelease(FwupdDevice *device)
 {
-    FwupdResource* res = createDevice(device);
-
     FwupdRelease *release = fwupd_device_get_release_default(device);
-    res->setId(QString::fromUtf8(fwupd_release_get_appstream_id(release)));
+    FwupdResource* res = new FwupdResource(device, QString::fromUtf8(fwupd_release_get_appstream_id(release)), this);
     res->setReleaseDetails(release);
 
     /* the same as we have already */
@@ -161,7 +131,7 @@ void FwupdBackend::addUpdates()
                     longdescription += QString::fromUtf8(fwupd_release_get_description(release)) + QLatin1Char('\n');
                 }
                 res->setDescription(longdescription);
-                addResourceToList(res);
+                addResource(res);
             }
         } else {
             if (g_error_matches(error2, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED)) {
@@ -237,113 +207,6 @@ FwupdResource* FwupdBackend::createApp(FwupdDevice *device)
     return app.take();
 }
 
-bool FwupdBackend::downloadFile(const QUrl &uri, const QString &filename)
-{
-    Q_ASSERT(QThread::currentThread() != QCoreApplication::instance()->thread());
-
-    QScopedPointer<QNetworkAccessManager> manager(new QNetworkAccessManager);
-    QEventLoop loop;
-    QTimer getTimer;
-    connect(&getTimer, &QTimer::timeout, &loop, &QEventLoop::quit);
-    connect(manager.data(), &QNetworkAccessManager::finished, &loop, &QEventLoop::quit);
-    QScopedPointer<QNetworkReply> reply(manager->get(QNetworkRequest(uri)));
-    getTimer.start(600000); // 60 Seconds TimeOout Period
-    loop.exec();
-    if (!reply)
-    {
-        return false;
-    } else if (QNetworkReply::NoError != reply->error() ) {
-        qWarning() << "error fetching" << uri;
-        return false;
-    } else if (reply->error() == QNetworkReply::NoError) {
-        QFile file(filename);
-        if (file.open(QIODevice::WriteOnly)) {
-            file.write(reply->readAll());
-        } else {
-            qWarning() << "Fwupd Error: Cannot Open File to write Data" << filename;
-        }
-    }
-    return true;
-}
-
-void FwupdBackend::refreshRemote(FwupdBackend* backend, FwupdRemote* remote, quint64 cacheAge, GCancellable *cancellable)
-{
-    if (!fwupd_remote_get_filename_cache_sig(remote))
-    {
-        qWarning() << "Fwupd Error: " << "Remote " << fwupd_remote_get_id(remote) << "has no cache signature!";
-        return;
-    }
-
-    /* check cache age */
-    if (cacheAge > 0)
-    {
-        const quint64 age = fwupd_remote_get_age(remote);
-        if (age < cacheAge)
-        {
-//             qDebug() << "Fwupd Info:" << fwupd_remote_get_id(remote) << "is only" << age << "seconds old, so ignoring refresh! ";
-            return;
-        }
-    }
-
-    const QString cacheId = QStringLiteral("fwupd/remotes.d/%1").arg(QString::fromUtf8(fwupd_remote_get_id(remote)));
-    const auto basenameSig = QString::fromUtf8(g_path_get_basename(fwupd_remote_get_filename_cache_sig(remote)));
-    const QString filenameSig = cacheFile(cacheId, basenameSig);
-
-    if (filenameSig.isEmpty())
-        return;
-
-    /* download the signature first*/
-    const QUrl urlSig(QString::fromUtf8(fwupd_remote_get_metadata_uri_sig(remote)));
-    const QString filenameSigTmp(filenameSig + QStringLiteral(".tmp"));
-
-    if (!downloadFile(urlSig, filenameSigTmp)) {
-        qWarning() << "failed to download" << urlSig;
-        return;
-    }
-    Q_ASSERT(QFile::exists(filenameSigTmp));
-
-    const auto checksum = fwupd_remote_get_checksum(remote);
-    const QCryptographicHash::Algorithm hashAlgorithm = gchecksumToQChryptographicHash()[fwupd_checksum_guess_kind(checksum)];
-    const QByteArray hash = getChecksum(filenameSigTmp, hashAlgorithm);
-
-    const QByteArray oldHash = getChecksum(filenameSig, hashAlgorithm);
-    if (oldHash == hash) {
-        qDebug() << "remote hasn't changed:" << fwupd_remote_get_id(remote);
-        QFile::remove(filenameSigTmp);
-        return;
-    }
-
-    QFile::remove(filenameSig);
-
-    if (!QFile::rename(filenameSigTmp, filenameSig)) {
-        QFile::remove(filenameSigTmp);
-        qWarning() << "Fwupd Error: cannot save remote signature" << filenameSigTmp << "to" << filenameSig;
-        return;
-    }
-    QFile::remove(filenameSigTmp);
-
-    const auto basename = QString::fromUtf8(g_path_get_basename(fwupd_remote_get_filename_cache(remote)));
-    const QString filename = cacheFile(cacheId, basename);
-
-    if (filename.isEmpty())
-        return;
-
-    qDebug() << "Fwupd Info: saving new firmware metadata to:" <<  filename;
-
-    const QUrl url(QString::fromUtf8(fwupd_remote_get_metadata_uri(remote)));
-    if (!downloadFile(url, filename))
-    {
-        qWarning() << "Fwupd Error: cannot download file:" << filename;
-        return;
-    }
-
-    g_autoptr(GError) error = nullptr;
-    if (!fwupd_client_update_metadata(backend->client, fwupd_remote_get_id(remote), filename.toUtf8().constData(), filenameSig.toUtf8().constData(), cancellable, &error))
-    {
-        backend->handleError(error);
-    }
-}
-
 void FwupdBackend::handleError(GError *perror)
 {
     //TODO: localise the error message
@@ -372,82 +235,111 @@ QString FwupdBackend::cacheFile(const QString &kind, const QString &basename)
     return cacheDir.filePath(kind + QLatin1Char('/') + basename);
 }
 
+static void fwupd_client_get_devices_cb (GObject */*source*/, GAsyncResult *res, gpointer user_data)
+{
+    FwupdBackend *helper = (FwupdBackend *) user_data;
+    g_autoptr(GError) error = nullptr;
+    auto array = fwupd_client_get_devices_finish(helper->client, res, &error);
+    if (!error)
+        helper->setDevices(array);
+    else
+        helper->handleError(error);
+}
+
+void FwupdBackend::setDevices(GPtrArray *devices)
+{
+    for(uint i = 0; devices && i < devices->len; i++) {
+        FwupdDevice *device = (FwupdDevice *) g_ptr_array_index(devices, i);
+
+        if (!fwupd_device_has_flag (device, FWUPD_DEVICE_FLAG_SUPPORTED))
+            continue;
+
+        g_autoptr(GError) error = nullptr;
+        g_autoptr(GPtrArray) releases = fwupd_client_get_releases(client, fwupd_device_get_id(device), m_cancellable, &error);
+
+        if (error) {
+            if (g_error_matches(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED)) {
+                qWarning() << "fwupd: Device not supported:" << fwupd_device_get_name(device) << error->message;
+                continue;
+            }
+            if (g_error_matches(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_FILE)) {
+                continue;
+            }
+
+            handleError(error);
+        }
+
+        auto res = new FwupdResource(device, this);
+        for (uint i=0; releases && i<releases->len; ++i) {
+            FwupdRelease *release = (FwupdRelease *)g_ptr_array_index(releases, i);
+            if (res->installedVersion().toUtf8() == fwupd_release_get_version(release)) {
+                res->setReleaseDetails(release);
+                break;
+            }
+        }
+        addResource(res);
+    }
+    g_ptr_array_unref(devices);
+
+    addUpdates();
+
+    m_fetching = false;
+    emit fetchingChanged();
+    emit initialized();
+}
+
+static void fwupd_client_get_remotes_cb (GObject */*source*/, GAsyncResult *res, gpointer user_data)
+{
+    FwupdBackend *helper = (FwupdBackend *) user_data;
+    g_autoptr(GError) error = nullptr;
+    auto array = fwupd_client_get_remotes_finish(helper->client, res, &error);
+    if (!error)
+        helper->setRemotes(array);
+    else
+        helper->handleError(error);
+}
+
+static void fwupd_client_refresh_remote_cb (GObject */*source*/, GAsyncResult *res, gpointer user_data)
+{
+    FwupdBackend *helper = (FwupdBackend *) user_data;
+    g_autoptr(GError) error = nullptr;
+    fwupd_client_refresh_remote_finish(helper->client, res, &error);
+    if (error)
+        helper->handleError(error);
+}
+
+void FwupdBackend::setRemotes(GPtrArray *remotes)
+{
+    for(uint i = 0; remotes && i < remotes->len; i++)
+    {
+        FwupdRemote *remote = (FwupdRemote *)g_ptr_array_index(remotes, i);
+        if (!fwupd_remote_get_enabled(remote))
+            continue;
+
+        if (fwupd_remote_get_kind(remote) == FWUPD_REMOTE_KIND_LOCAL)
+            continue;
+
+        fwupd_client_refresh_remote_async(client, remote, m_cancellable, fwupd_client_refresh_remote_cb, this);
+    }
+}
+
 void FwupdBackend::checkForUpdates()
 {
     if (m_fetching)
         return;
 
-    auto fw = new QFutureWatcher<GPtrArray*>(this);
-    connect(fw, &QFutureWatcher<GPtrArray*>::finished, this, [this, fw]() {
-        m_fetching = true;
-        emit fetchingChanged();
+    g_autoptr(GError) error = nullptr;
 
-        auto devices = fw->result();
-        for(uint i = 0; devices && i < devices->len; i++) {
-            FwupdDevice *device = (FwupdDevice *) g_ptr_array_index(devices, i);
+    if (!fwupd_client_connect (client, m_cancellable, &error)) {
+        handleError(error);
+        return;
+    }
 
-            if (!fwupd_device_has_flag (device, FWUPD_DEVICE_FLAG_SUPPORTED))
-                continue;
+    m_fetching = true;
+    emit fetchingChanged();
 
-            g_autoptr(GError) error = nullptr;
-            g_autoptr(GPtrArray) releases = fwupd_client_get_releases(client, fwupd_device_get_id(device), m_cancellable, &error);
-
-            if (error) {
-                if (g_error_matches(error, FWUPD_ERROR, FWUPD_ERROR_NOT_SUPPORTED)) {
-                    qWarning() << "fwupd: Device not supported:" << fwupd_device_get_name(device) << error->message;
-                    continue;
-                }
-                if (g_error_matches(error, FWUPD_ERROR, FWUPD_ERROR_INVALID_FILE)) {
-                    continue;
-                }
-
-                handleError(error);
-            }
-
-            auto res = createDevice(device);
-            for (uint i=0; releases && i<releases->len; ++i) {
-                FwupdRelease *release = (FwupdRelease *)g_ptr_array_index(releases, i);
-                if (res->installedVersion().toUtf8() == fwupd_release_get_version(release)) {
-                    res->setReleaseDetails(release);
-                    break;
-                }
-            }
-            addResourceToList(res);
-        }
-        g_ptr_array_unref(devices);
-
-        addUpdates();
-
-        m_fetching = false;
-        emit fetchingChanged();
-        emit initialized();
-        fw->deleteLater();
-    });
-    fw->setFuture(QtConcurrent::run(&m_threadPool, [this] () -> GPtrArray*
-        {
-            const uint cacheAge = (24*60*60); // Check for updates every day
-            g_autoptr(GError) error = nullptr;
-
-            /* get devices */
-            GPtrArray* devices = fwupd_client_get_devices(client, m_cancellable, &error);
-            handleError(error);
-
-            g_autoptr(GPtrArray) remotes = fwupd_client_get_remotes(client, m_cancellable, &error);
-            handleError(error);
-            for(uint i = 0; remotes && i < remotes->len; i++)
-            {
-                FwupdRemote *remote = (FwupdRemote *)g_ptr_array_index(remotes, i);
-                if (!fwupd_remote_get_enabled(remote))
-                    continue;
-
-                if (fwupd_remote_get_kind(remote) == FWUPD_REMOTE_KIND_LOCAL)
-                    continue;
-
-                refreshRemote(this, remote, cacheAge, m_cancellable);
-            }
-            return devices;
-        }
-    ));
+    fwupd_client_get_devices_async(client, m_cancellable, fwupd_client_get_devices_cb, this);
+    fwupd_client_get_remotes_async(client, m_cancellable, fwupd_client_get_remotes_cb, this);
 }
 
 int FwupdBackend::updatesCount() const
@@ -555,7 +447,7 @@ ResultsStream* FwupdBackend::resourceForFile(const QUrl& path)
                 FwupdResource* app_ = createRelease(device);
                 app_->setState(AbstractResource::None);
             }
-            addResourceToList(app);
+            addResource(app);
             connect(app, &FwupdResource::stateChanged, this, &FwupdBackend::updatesCountChanged);
             return new ResultsStream(QStringLiteral("FwupdStream-file"), {app});
         }
